@@ -29,6 +29,13 @@
 #include <sys/sysctl.h>
 #include <sys/user.h>
 #endif
+#ifdef HAVE_NETBSD
+#include <sys/param.h>
+#include <sys/sysctl.h>
+#endif
+#ifdef HAVE_OPENBSD
+#include <sys/sysctl.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -86,8 +93,13 @@ static guint64 get_start_time_for_pid (gint    pid,
 static gint _polkit_unix_process_get_owner (PolkitUnixProcess  *process,
                                             GError            **error);
 
-#ifdef HAVE_FREEBSD
-static gboolean get_kinfo_proc (gint pid, struct kinfo_proc *p);
+#if defined(HAVE_FREEBSD) || defined(HAVE_NETBSD) || defined(HAVE_OPENBSD)
+static gboolean get_kinfo_proc (gint pid,
+#if defined(HAVE_NETBSD)
+                                struct kinfo_proc2 *p);
+#else
+                                struct kinfo_proc *p);
+#endif
 #endif
 
 G_DEFINE_TYPE_WITH_CODE (PolkitUnixProcess, polkit_unix_process, G_TYPE_OBJECT,
@@ -554,12 +566,45 @@ get_kinfo_proc (pid_t pid, struct kinfo_proc *p)
 }
 #endif
 
+#if defined(HAVE_NETBSD) || defined(HAVE_OPENBSD)
+static gboolean
+get_kinfo_proc (gint pid,
+#ifdef HAVE_NETBSD
+                struct kinfo_proc2 *p)
+#else
+                struct kinfo_proc *p)
+#endif
+{
+  int name[6];
+  u_int namelen;
+  size_t sz;
+
+  sz = sizeof(*p);
+  namelen = 0;
+  name[namelen++] = CTL_KERN;
+#ifdef HAVE_NETBSD
+  name[namelen++] = KERN_PROC2;
+#else
+  name[namelen++] = KERN_PROC;
+#endif
+  name[namelen++] = KERN_PROC_PID;
+  name[namelen++] = pid;
+  name[namelen++] = sz;
+  name[namelen++] = 1;
+
+  if (sysctl (name, namelen, p, &sz, NULL, 0) == -1)
+    return FALSE;
+
+  return TRUE;
+}
+#endif
+
 static guint64
 get_start_time_for_pid (pid_t    pid,
                         GError **error)
 {
   guint64 start_time;
-#ifndef HAVE_FREEBSD
+#if !defined(HAVE_FREEBSD) && !defined(HAVE_NETBSD) && !defined(HAVE_OPENBSD)
   gchar *filename;
   gchar *contents;
   size_t length;
@@ -631,8 +676,12 @@ get_start_time_for_pid (pid_t    pid,
  out:
   g_free (filename);
   g_free (contents);
+#else
+#ifdef HAVE_NETBSD
+  struct kinfo_proc2 p;
 #else
   struct kinfo_proc p;
+#endif
 
   start_time = 0;
 
@@ -647,7 +696,11 @@ get_start_time_for_pid (pid_t    pid,
       goto out;
     }
 
+#ifdef HAVE_FREEBSD
   start_time = (guint64) p.ki_start.tv_sec;
+#else
+  start_time = (guint64) p.p_ustart_sec;
+#endif
 
 out:
 #endif
@@ -662,8 +715,10 @@ _polkit_unix_process_get_owner (PolkitUnixProcess  *process,
   gint result;
   gchar *contents;
   gchar **lines;
-#ifdef HAVE_FREEBSD
+#if defined(HAVE_FREEBSD) || defined(HAVE_OPENBSD)
   struct kinfo_proc p;
+#elif defined(HAVE_NETBSD)
+  struct kinfo_proc2 p;
 #else
   gchar filename[64];
   guint n;
@@ -676,7 +731,7 @@ _polkit_unix_process_get_owner (PolkitUnixProcess  *process,
   lines = NULL;
   contents = NULL;
 
-#ifdef HAVE_FREEBSD
+#if defined(HAVE_FREEBSD) || defined(HAVE_NETBSD) || defined(HAVE_OPENBSD)
   if (get_kinfo_proc (process->pid, &p) == 0)
     {
       g_set_error (error,
@@ -688,7 +743,11 @@ _polkit_unix_process_get_owner (PolkitUnixProcess  *process,
       goto out;
     }
 
+#if defined(HAVE_FREEBSD)
   result = p.ki_uid;
+#else
+  result = p.p_uid;
+#endif
 #else
 
   /* see 'man proc' for layout of the status file
